/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * Asset defines a financial product such as stocks, bonds, or
 * mutual funds etc 
 * 
 * @version $Revision$ $Date$
 */
public class Asset implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * Each asset in the assetpool has an unique identifier.
     *  
     */
    private int _assetID;

    /**
     * keeps track of state for field: _assetID
     */
    private boolean _has_assetID;

    /**
     * Field _date.
     */
    private java.util.Date _date;

    /**
     * The price of the asset at time = 0. (the time at which the
     * price of the option needs to be computed.)
     *  
     */
    private double _startPrice;

    /**
     * keeps track of state for field: _startPrice
     */
    private boolean _has_startPrice;

    /**
     * A variable in option pricing formulas showing the extent to
     *  which the return of the underlying asset will fluctuate
     * between now and the
     *  option's expiration. Volatility, as expressed as a
     * percentage coefficient
     *  within option-pricing formulas, arises from daily trading
     * activities. How
     *  volatility is measured will affect the value of the
     * coefficient used. In
     *  other words, volatility refers to the amount of uncertainty
     * or risk about
     *  the size of changes in a security's value. A higher
     * volatility means that a
     *  security's value can potentially be spread out over a
     * larger range of
     *  values. This means that the price of the security can
     * change dramatically
     *  over a short time period in either direction. A lower
     * volatility means that
     *  a security's value does not fluctuate dramatically, but
     * changes in value at
     *  a steady pace over a period of time. One measure of the
     * relative volatility
     *  of a particular stock to the market is its beta. A beta
     * approximates the
     *  overall volatility of a security's returns against the
     * returns of a relevant
     *  benchmark (usually the SP500 is used). For example, a stock
     * with a beta
     *  value of 1.1 has historically moved 110% for every 100%
     * move in the
     *  benchmark, based on price level. Conversely, a stock with a
     * beta of .9 has
     *  historically moved 90% for every 100% move in the
     * underlying index.
     *  (Investopedia.com ) 
     */
    private double _volatility;

    /**
     * keeps track of state for field: _volatility
     */
    private boolean _has_volatility;

    /**
     * When a company pays a share of the profit to existing
     *  shareholders. This share of profit may be in cash or
     * options.
     *  (optiontradingpedia) 
     */
    private double _dividend = 0.0;

    /**
     * keeps track of state for field: _dividend
     */
    private boolean _has_dividend;

    //----------------/
    //- Constructors -/
    //----------------/

    public Asset() {
        super();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     */
    public void deleteAssetID() {
        this._has_assetID = false;
    }

    /**
     */
    public void deleteDividend() {
        this._has_dividend = false;
    }

    /**
     */
    public void deleteStartPrice() {
        this._has_startPrice = false;
    }

    /**
     */
    public void deleteVolatility() {
        this._has_volatility = false;
    }

    /**
     * Returns the value of field 'assetID'. The field 'assetID'
     * has the following description: Each asset in the assetpool
     * has an unique identifier.
     *  
     * 
     * @return the value of field 'AssetID'.
     */
    public int getAssetID() {
        return this._assetID;
    }

    /**
     * Returns the value of field 'date'.
     * 
     * @return the value of field 'Date'.
     */
    public java.util.Date getDate() {
        return this._date;
    }

    /**
     * Returns the value of field 'dividend'. The field 'dividend'
     * has the following description: When a company pays a share
     * of the profit to existing
     *  shareholders. This share of profit may be in cash or
     * options.
     *  (optiontradingpedia) 
     * 
     * @return the value of field 'Dividend'.
     */
    public double getDividend() {
        return this._dividend;
    }

    /**
     * Returns the value of field 'startPrice'. The field
     * 'startPrice' has the following description: The price of the
     * asset at time = 0. (the time at which the price of the
     * option needs to be computed.)
     *  
     * 
     * @return the value of field 'StartPrice'.
     */
    public double getStartPrice() {
        return this._startPrice;
    }

    /**
     * Returns the value of field 'volatility'. The field
     * 'volatility' has the following description: A variable in
     * option pricing formulas showing the extent to
     *  which the return of the underlying asset will fluctuate
     * between now and the
     *  option's expiration. Volatility, as expressed as a
     * percentage coefficient
     *  within option-pricing formulas, arises from daily trading
     * activities. How
     *  volatility is measured will affect the value of the
     * coefficient used. In
     *  other words, volatility refers to the amount of uncertainty
     * or risk about
     *  the size of changes in a security's value. A higher
     * volatility means that a
     *  security's value can potentially be spread out over a
     * larger range of
     *  values. This means that the price of the security can
     * change dramatically
     *  over a short time period in either direction. A lower
     * volatility means that
     *  a security's value does not fluctuate dramatically, but
     * changes in value at
     *  a steady pace over a period of time. One measure of the
     * relative volatility
     *  of a particular stock to the market is its beta. A beta
     * approximates the
     *  overall volatility of a security's returns against the
     * returns of a relevant
     *  benchmark (usually the SP500 is used). For example, a stock
     * with a beta
     *  value of 1.1 has historically moved 110% for every 100%
     * move in the
     *  benchmark, based on price level. Conversely, a stock with a
     * beta of .9 has
     *  historically moved 90% for every 100% move in the
     * underlying index.
     *  (Investopedia.com ) 
     * 
     * @return the value of field 'Volatility'.
     */
    public double getVolatility() {
        return this._volatility;
    }

    /**
     * Method hasAssetID.
     * 
     * @return true if at least one AssetID has been added
     */
    public boolean hasAssetID() {
        return this._has_assetID;
    }

    /**
     * Method hasDividend.
     * 
     * @return true if at least one Dividend has been added
     */
    public boolean hasDividend() {
        return this._has_dividend;
    }

    /**
     * Method hasStartPrice.
     * 
     * @return true if at least one StartPrice has been added
     */
    public boolean hasStartPrice() {
        return this._has_startPrice;
    }

    /**
     * Method hasVolatility.
     * 
     * @return true if at least one Volatility has been added
     */
    public boolean hasVolatility() {
        return this._has_volatility;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     * Sets the value of field 'assetID'. The field 'assetID' has
     * the following description: Each asset in the assetpool has
     * an unique identifier.
     *  
     * 
     * @param assetID the value of field 'assetID'.
     */
    public void setAssetID(final int assetID) {
        this._assetID = assetID;
        this._has_assetID = true;
    }

    /**
     * Sets the value of field 'date'.
     * 
     * @param date the value of field 'date'.
     */
    public void setDate(final java.util.Date date) {
        this._date = date;
    }

    /**
     * Sets the value of field 'dividend'. The field 'dividend' has
     * the following description: When a company pays a share of
     * the profit to existing
     *  shareholders. This share of profit may be in cash or
     * options.
     *  (optiontradingpedia) 
     * 
     * @param dividend the value of field 'dividend'.
     */
    public void setDividend(final double dividend) {
        this._dividend = dividend;
        this._has_dividend = true;
    }

    /**
     * Sets the value of field 'startPrice'. The field 'startPrice'
     * has the following description: The price of the asset at
     * time = 0. (the time at which the price of the option needs
     * to be computed.)
     *  
     * 
     * @param startPrice the value of field 'startPrice'.
     */
    public void setStartPrice(final double startPrice) {
        this._startPrice = startPrice;
        this._has_startPrice = true;
    }

    /**
     * Sets the value of field 'volatility'. The field 'volatility'
     * has the following description: A variable in option pricing
     * formulas showing the extent to
     *  which the return of the underlying asset will fluctuate
     * between now and the
     *  option's expiration. Volatility, as expressed as a
     * percentage coefficient
     *  within option-pricing formulas, arises from daily trading
     * activities. How
     *  volatility is measured will affect the value of the
     * coefficient used. In
     *  other words, volatility refers to the amount of uncertainty
     * or risk about
     *  the size of changes in a security's value. A higher
     * volatility means that a
     *  security's value can potentially be spread out over a
     * larger range of
     *  values. This means that the price of the security can
     * change dramatically
     *  over a short time period in either direction. A lower
     * volatility means that
     *  a security's value does not fluctuate dramatically, but
     * changes in value at
     *  a steady pace over a period of time. One measure of the
     * relative volatility
     *  of a particular stock to the market is its beta. A beta
     * approximates the
     *  overall volatility of a security's returns against the
     * returns of a relevant
     *  benchmark (usually the SP500 is used). For example, a stock
     * with a beta
     *  value of 1.1 has historically moved 110% for every 100%
     * move in the
     *  benchmark, based on price level. Conversely, a stock with a
     * beta of .9 has
     *  historically moved 90% for every 100% move in the
     * underlying index.
     *  (Investopedia.com ) 
     * 
     * @param volatility the value of field 'volatility'.
     */
    public void setVolatility(final double volatility) {
        this._volatility = volatility;
        this._has_volatility = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.Asset
     */
    public static plugtest.data.Asset unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.Asset) Unmarshaller.unmarshal(plugtest.data.Asset.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
