/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * In the assetpool the assets are indexed linearly rather
 *  than asset indices. Note that the asset indices are numbered
 * from 0 to N.
 *  
 * 
 * @version $Revision$ $Date$
 */
public class AssetPool implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * Asset defines a financial product such as stocks, bonds, or
     * mutual funds etc 
     */
    private java.util.List _assetList;

    //----------------/
    //- Constructors -/
    //----------------/

    public AssetPool() {
        super();
        this._assetList = new java.util.ArrayList();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * 
     * 
     * @param vAsset
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addAsset(final plugtest.data.Asset vAsset) throws java.lang.IndexOutOfBoundsException {
        this._assetList.add(vAsset);
    }

    /**
     * 
     * 
     * @param index
     * @param vAsset
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addAsset(final int index, final plugtest.data.Asset vAsset)
            throws java.lang.IndexOutOfBoundsException {
        this._assetList.add(index, vAsset);
    }

    /**
     * Method enumerateAsset.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateAsset() {
        return java.util.Collections.enumeration(this._assetList);
    }

    /**
     * Method getAsset.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the plugtest.data.Asset at the given
     * index
     */
    public plugtest.data.Asset getAsset(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._assetList.size()) {
            throw new IndexOutOfBoundsException("getAsset: Index value '" + index + "' not in range [0.." +
                (this._assetList.size() - 1) + "]");
        }

        return (plugtest.data.Asset) _assetList.get(index);
    }

    /**
     * Method getAsset.Returns the contents of the collection in an
     * Array.  <p>Note:  Just in case the collection contents are
     * changing in another thread, we pass a 0-length Array of the
     * correct type into the API call.  This way we <i>know</i>
     * that the Array returned is of exactly the correct length.
     * 
     * @return this collection as an Array
     */
    public plugtest.data.Asset[] getAsset() {
        plugtest.data.Asset[] array = new plugtest.data.Asset[0];
        return (plugtest.data.Asset[]) this._assetList.toArray(array);
    }

    /**
     * Method getAssetCount.
     * 
     * @return the size of this collection
     */
    public int getAssetCount() {
        return this._assetList.size();
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * Method iterateAsset.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateAsset() {
        return this._assetList.iterator();
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     */
    public void removeAllAsset() {
        this._assetList.clear();
    }

    /**
     * Method removeAsset.
     * 
     * @param vAsset
     * @return true if the object was removed from the collection.
     */
    public boolean removeAsset(final plugtest.data.Asset vAsset) {
        boolean removed = _assetList.remove(vAsset);
        return removed;
    }

    /**
     * Method removeAssetAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public plugtest.data.Asset removeAssetAt(final int index) {
        java.lang.Object obj = this._assetList.remove(index);
        return (plugtest.data.Asset) obj;
    }

    /**
     * 
     * 
     * @param index
     * @param vAsset
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setAsset(final int index, final plugtest.data.Asset vAsset)
            throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._assetList.size()) {
            throw new IndexOutOfBoundsException("setAsset: Index value '" + index + "' not in range [0.." +
                (this._assetList.size() - 1) + "]");
        }

        this._assetList.set(index, vAsset);
    }

    /**
     * 
     * 
     * @param vAssetArray
     */
    public void setAsset(final plugtest.data.Asset[] vAssetArray) {
        //-- copy array
        _assetList.clear();

        for (int i = 0; i < vAssetArray.length; i++) {
            this._assetList.add(vAssetArray[i]);
        }
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.AssetPool
     */
    public static plugtest.data.AssetPool unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.AssetPool) Unmarshaller.unmarshal(plugtest.data.AssetPool.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
