/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * defines the constant market parameters that to be used for all
 * the testcases.
 *  
 * 
 * @version $Revision$ $Date$
 */
public class BasketParams implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * risk free interest rate (annulized continuously compounded
     * rate on a safe asset with the same maturity as the
     * expiration of option); usually money market rate for a
     * maturity equal to the option's maturity
     *  
     */
    private double _riskFreeRate;

    /**
     * keeps track of state for field: _riskFreeRate
     */
    private boolean _has_riskFreeRate;

    /**
     * The time intervals per year. This parameter gives the
     * discretization for the computational algorithm.
     *  
     */
    private double _timeIntervalsPerYear;

    /**
     * keeps track of state for field: _timeIntervalsPerYear
     */
    private boolean _has_timeIntervalsPerYear;

    /**
     * change in time, required for computing greeks 
     */
    private double _epsilonT;

    /**
     * keeps track of state for field: _epsilonT
     */
    private boolean _has_epsilonT;

    /**
     * change in price, required for computing greeks 
     */
    private double _epsilonS;

    /**
     * keeps track of state for field: _epsilonS
     */
    private boolean _has_epsilonS;

    /**
     * change in interest rate, required for computing greeks 
     */
    private double _epsilonR;

    /**
     * keeps track of state for field: _epsilonR
     */
    private boolean _has_epsilonR;

    //----------------/
    //- Constructors -/
    //----------------/

    public BasketParams() {
        super();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     */
    public void deleteEpsilonR() {
        this._has_epsilonR = false;
    }

    /**
     */
    public void deleteEpsilonS() {
        this._has_epsilonS = false;
    }

    /**
     */
    public void deleteEpsilonT() {
        this._has_epsilonT = false;
    }

    /**
     */
    public void deleteRiskFreeRate() {
        this._has_riskFreeRate = false;
    }

    /**
     */
    public void deleteTimeIntervalsPerYear() {
        this._has_timeIntervalsPerYear = false;
    }

    /**
     * Returns the value of field 'epsilonR'. The field 'epsilonR'
     * has the following description: change in interest rate,
     * required for computing greeks 
     * 
     * @return the value of field 'EpsilonR'.
     */
    public double getEpsilonR() {
        return this._epsilonR;
    }

    /**
     * Returns the value of field 'epsilonS'. The field 'epsilonS'
     * has the following description: change in price, required for
     * computing greeks 
     * 
     * @return the value of field 'EpsilonS'.
     */
    public double getEpsilonS() {
        return this._epsilonS;
    }

    /**
     * Returns the value of field 'epsilonT'. The field 'epsilonT'
     * has the following description: change in time, required for
     * computing greeks 
     * 
     * @return the value of field 'EpsilonT'.
     */
    public double getEpsilonT() {
        return this._epsilonT;
    }

    /**
     * Returns the value of field 'riskFreeRate'. The field
     * 'riskFreeRate' has the following description: risk free
     * interest rate (annulized continuously compounded rate on a
     * safe asset with the same maturity as the expiration of
     * option); usually money market rate for a maturity equal to
     * the option's maturity
     *  
     * 
     * @return the value of field 'RiskFreeRate'.
     */
    public double getRiskFreeRate() {
        return this._riskFreeRate;
    }

    /**
     * Returns the value of field 'timeIntervalsPerYear'. The field
     * 'timeIntervalsPerYear' has the following description: The
     * time intervals per year. This parameter gives the
     * discretization for the computational algorithm.
     *  
     * 
     * @return the value of field 'TimeIntervalsPerYear'.
     */
    public double getTimeIntervalsPerYear() {
        return this._timeIntervalsPerYear;
    }

    /**
     * Method hasEpsilonR.
     * 
     * @return true if at least one EpsilonR has been added
     */
    public boolean hasEpsilonR() {
        return this._has_epsilonR;
    }

    /**
     * Method hasEpsilonS.
     * 
     * @return true if at least one EpsilonS has been added
     */
    public boolean hasEpsilonS() {
        return this._has_epsilonS;
    }

    /**
     * Method hasEpsilonT.
     * 
     * @return true if at least one EpsilonT has been added
     */
    public boolean hasEpsilonT() {
        return this._has_epsilonT;
    }

    /**
     * Method hasRiskFreeRate.
     * 
     * @return true if at least one RiskFreeRate has been added
     */
    public boolean hasRiskFreeRate() {
        return this._has_riskFreeRate;
    }

    /**
     * Method hasTimeIntervalsPerYear.
     * 
     * @return true if at least one TimeIntervalsPerYear has been
     * added
     */
    public boolean hasTimeIntervalsPerYear() {
        return this._has_timeIntervalsPerYear;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     * Sets the value of field 'epsilonR'. The field 'epsilonR' has
     * the following description: change in interest rate, required
     * for computing greeks 
     * 
     * @param epsilonR the value of field 'epsilonR'.
     */
    public void setEpsilonR(final double epsilonR) {
        this._epsilonR = epsilonR;
        this._has_epsilonR = true;
    }

    /**
     * Sets the value of field 'epsilonS'. The field 'epsilonS' has
     * the following description: change in price, required for
     * computing greeks 
     * 
     * @param epsilonS the value of field 'epsilonS'.
     */
    public void setEpsilonS(final double epsilonS) {
        this._epsilonS = epsilonS;
        this._has_epsilonS = true;
    }

    /**
     * Sets the value of field 'epsilonT'. The field 'epsilonT' has
     * the following description: change in time, required for
     * computing greeks 
     * 
     * @param epsilonT the value of field 'epsilonT'.
     */
    public void setEpsilonT(final double epsilonT) {
        this._epsilonT = epsilonT;
        this._has_epsilonT = true;
    }

    /**
     * Sets the value of field 'riskFreeRate'. The field
     * 'riskFreeRate' has the following description: risk free
     * interest rate (annulized continuously compounded rate on a
     * safe asset with the same maturity as the expiration of
     * option); usually money market rate for a maturity equal to
     * the option's maturity
     *  
     * 
     * @param riskFreeRate the value of field 'riskFreeRate'.
     */
    public void setRiskFreeRate(final double riskFreeRate) {
        this._riskFreeRate = riskFreeRate;
        this._has_riskFreeRate = true;
    }

    /**
     * Sets the value of field 'timeIntervalsPerYear'. The field
     * 'timeIntervalsPerYear' has the following description: The
     * time intervals per year. This parameter gives the
     * discretization for the computational algorithm.
     *  
     * 
     * @param timeIntervalsPerYear the value of field
     * 'timeIntervalsPerYear'.
     */
    public void setTimeIntervalsPerYear(final double timeIntervalsPerYear) {
        this._timeIntervalsPerYear = timeIntervalsPerYear;
        this._has_timeIntervalsPerYear = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.BasketParams
     */
    public static plugtest.data.BasketParams unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.BasketParams) Unmarshaller.unmarshal(plugtest.data.BasketParams.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
