/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * Defines the correlation of each asset with the other assets in
 * the asset pool
 *  
 * 
 * @version $Revision$ $Date$
 */
public class Correlation implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * Field _assetID.
     */
    private int _assetID;

    /**
     * keeps track of state for field: _assetID
     */
    private boolean _has_assetID;

    /**
     * This vector defines the correlation of asset with the asset
     * id "AssetID" with the rest of the assets in the global asset
     * pool.
     *  
     */
    private java.util.List _valuesList;

    //----------------/
    //- Constructors -/
    //----------------/

    public Correlation() {
        super();
        this._valuesList = new java.util.ArrayList();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * 
     * 
     * @param vValues
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addValues(final double vValues) throws java.lang.IndexOutOfBoundsException {
        this._valuesList.add(new java.lang.Double(vValues));
    }

    /**
     * 
     * 
     * @param index
     * @param vValues
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addValues(final int index, final double vValues) throws java.lang.IndexOutOfBoundsException {
        this._valuesList.add(index, new java.lang.Double(vValues));
    }

    /**
     */
    public void deleteAssetID() {
        this._has_assetID = false;
    }

    /**
     * Method enumerateValues.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateValues() {
        return java.util.Collections.enumeration(this._valuesList);
    }

    /**
     * Returns the value of field 'assetID'.
     * 
     * @return the value of field 'AssetID'.
     */
    public int getAssetID() {
        return this._assetID;
    }

    /**
     * Method getValues.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the double at the given index
     */
    public double getValues(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._valuesList.size()) {
            throw new IndexOutOfBoundsException("getValues: Index value '" + index + "' not in range [0.." +
                (this._valuesList.size() - 1) + "]");
        }

        return ((java.lang.Double) _valuesList.get(index)).doubleValue();
    }

    /**
     * Method getValues.Returns the contents of the collection in
     * an Array.  
     * 
     * @return this collection as an Array
     */
    public double[] getValues() {
        int size = this._valuesList.size();
        double[] array = new double[size];
        java.util.Iterator iter = _valuesList.iterator();
        for (int index = 0; index < size; index++) {
            array[index] = ((java.lang.Double) iter.next()).doubleValue();
        }
        return array;
    }

    /**
     * Method getValuesCount.
     * 
     * @return the size of this collection
     */
    public int getValuesCount() {
        return this._valuesList.size();
    }

    /**
     * Method hasAssetID.
     * 
     * @return true if at least one AssetID has been added
     */
    public boolean hasAssetID() {
        return this._has_assetID;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * Method iterateValues.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateValues() {
        return this._valuesList.iterator();
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     */
    public void removeAllValues() {
        this._valuesList.clear();
    }

    /**
     * Method removeValues.
     * 
     * @param vValues
     * @return true if the object was removed from the collection.
     */
    public boolean removeValues(final double vValues) {
        boolean removed = _valuesList.remove(new java.lang.Double(vValues));
        return removed;
    }

    /**
     * Method removeValuesAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public double removeValuesAt(final int index) {
        java.lang.Object obj = this._valuesList.remove(index);
        return ((java.lang.Double) obj).doubleValue();
    }

    /**
     * Sets the value of field 'assetID'.
     * 
     * @param assetID the value of field 'assetID'.
     */
    public void setAssetID(final int assetID) {
        this._assetID = assetID;
        this._has_assetID = true;
    }

    /**
     * 
     * 
     * @param index
     * @param vValues
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setValues(final int index, final double vValues) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._valuesList.size()) {
            throw new IndexOutOfBoundsException("setValues: Index value '" + index + "' not in range [0.." +
                (this._valuesList.size() - 1) + "]");
        }

        this._valuesList.set(index, new java.lang.Double(vValues));
    }

    /**
     * 
     * 
     * @param vValuesArray
     */
    public void setValues(final double[] vValuesArray) {
        //-- copy array
        _valuesList.clear();

        for (int i = 0; i < vValuesArray.length; i++) {
            this._valuesList.add(new java.lang.Double(vValuesArray[i]));
        }
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.Correlation
     */
    public static plugtest.data.Correlation unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.Correlation) Unmarshaller.unmarshal(plugtest.data.Correlation.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
