/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * The correlation matrix is stored in the following format: an
 *  asset index followed by the correlation values with asset
 * indices from 0 to
 *  N (including the correlation with its own which is 1.0) 
 * 
 * @version $Revision$ $Date$
 */
public class CorrelationMatrix implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * Defines the correlation of each asset with the other assets
     * in the asset pool
     *  
     */
    private java.util.List _correlationList;

    //----------------/
    //- Constructors -/
    //----------------/

    public CorrelationMatrix() {
        super();
        this._correlationList = new java.util.ArrayList();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * 
     * 
     * @param vCorrelation
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addCorrelation(final plugtest.data.Correlation vCorrelation)
            throws java.lang.IndexOutOfBoundsException {
        this._correlationList.add(vCorrelation);
    }

    /**
     * 
     * 
     * @param index
     * @param vCorrelation
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addCorrelation(final int index, final plugtest.data.Correlation vCorrelation)
            throws java.lang.IndexOutOfBoundsException {
        this._correlationList.add(index, vCorrelation);
    }

    /**
     * Method enumerateCorrelation.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateCorrelation() {
        return java.util.Collections.enumeration(this._correlationList);
    }

    /**
     * Method getCorrelation.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the plugtest.data.Correlation at the
     * given index
     */
    public plugtest.data.Correlation getCorrelation(final int index)
            throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._correlationList.size()) {
            throw new IndexOutOfBoundsException("getCorrelation: Index value '" + index +
                "' not in range [0.." + (this._correlationList.size() - 1) + "]");
        }

        return (plugtest.data.Correlation) _correlationList.get(index);
    }

    /**
     * Method getCorrelation.Returns the contents of the collection
     * in an Array.  <p>Note:  Just in case the collection contents
     * are changing in another thread, we pass a 0-length Array of
     * the correct type into the API call.  This way we <i>know</i>
     * that the Array returned is of exactly the correct length.
     * 
     * @return this collection as an Array
     */
    public plugtest.data.Correlation[] getCorrelation() {
        plugtest.data.Correlation[] array = new plugtest.data.Correlation[0];
        return (plugtest.data.Correlation[]) this._correlationList.toArray(array);
    }

    /**
     * Method getCorrelationCount.
     * 
     * @return the size of this collection
     */
    public int getCorrelationCount() {
        return this._correlationList.size();
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * Method iterateCorrelation.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateCorrelation() {
        return this._correlationList.iterator();
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     */
    public void removeAllCorrelation() {
        this._correlationList.clear();
    }

    /**
     * Method removeCorrelation.
     * 
     * @param vCorrelation
     * @return true if the object was removed from the collection.
     */
    public boolean removeCorrelation(final plugtest.data.Correlation vCorrelation) {
        boolean removed = _correlationList.remove(vCorrelation);
        return removed;
    }

    /**
     * Method removeCorrelationAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public plugtest.data.Correlation removeCorrelationAt(final int index) {
        java.lang.Object obj = this._correlationList.remove(index);
        return (plugtest.data.Correlation) obj;
    }

    /**
     * 
     * 
     * @param index
     * @param vCorrelation
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setCorrelation(final int index, final plugtest.data.Correlation vCorrelation)
            throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._correlationList.size()) {
            throw new IndexOutOfBoundsException("setCorrelation: Index value '" + index +
                "' not in range [0.." + (this._correlationList.size() - 1) + "]");
        }

        this._correlationList.set(index, vCorrelation);
    }

    /**
     * 
     * 
     * @param vCorrelationArray
     */
    public void setCorrelation(final plugtest.data.Correlation[] vCorrelationArray) {
        //-- copy array
        _correlationList.clear();

        for (int i = 0; i < vCorrelationArray.length; i++) {
            this._correlationList.add(vCorrelationArray[i]);
        }
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.CorrelationMatrix
     */
    public static plugtest.data.CorrelationMatrix unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.CorrelationMatrix) Unmarshaller.unmarshal(
                plugtest.data.CorrelationMatrix.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
