/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * In mathematical finance, the Greeks are the quantities
 * representing
 *  the market sensitivities of derivatives such as options. Each
 * "Greek" measures a
 *  different aspect of the risk in an option position, and
 * corresponds to a parameter
 *  on which the value of an instrument or portfolio of financial
 * instruments is
 *  dependent. The name is used because the parameters are often
 * denoted by Greek
 *  letters. (Wikipedia) 
 * 
 * @version $Revision$ $Date$
 */
public class Greeks implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * represents the rate of change between the option's price and
     *  the underlying asset's price - in other words, price
     * sensitivity. The deltas to be computed for each asset. 
     *  
     */
    private java.util.List _deltaList;

    /**
     * represents the rate of change between an option portfolio's
     * delta and the underlying asset's price - in other words,
     * second-order time price sensitivity. The gammas of each 
     *  
     */
    private java.util.List _gammaList;

    /**
     * represents the rate of change between an option portfolio
     * and time, or time sensitivity.
     *  
     */
    private double _theta;

    /**
     * keeps track of state for field: _theta
     */
    private boolean _has_theta;

    /**
     * represents the rate of change between an option portfolio's
     * value and the interest rate, or sensitivity to the interest
     * rate.
     *  
     */
    private double _rho;

    /**
     * keeps track of state for field: _rho
     */
    private boolean _has_rho;

    /**
     * measures third order sensitivity to price. The speed is the
     * third derivative of the value function with respect to the
     * underlying price. The speed for each asset must be computed.
     * 
     *  
     */
    private java.util.List _speedList;

    //----------------/
    //- Constructors -/
    //----------------/

    public Greeks() {
        super();
        this._deltaList = new java.util.ArrayList();
        this._gammaList = new java.util.ArrayList();
        this._speedList = new java.util.ArrayList();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * 
     * 
     * @param vDelta
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addDelta(final double vDelta) throws java.lang.IndexOutOfBoundsException {
        this._deltaList.add(new java.lang.Double(vDelta));
    }

    /**
     * 
     * 
     * @param index
     * @param vDelta
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addDelta(final int index, final double vDelta) throws java.lang.IndexOutOfBoundsException {
        this._deltaList.add(index, new java.lang.Double(vDelta));
    }

    /**
     * 
     * 
     * @param vGamma
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addGamma(final double vGamma) throws java.lang.IndexOutOfBoundsException {
        this._gammaList.add(new java.lang.Double(vGamma));
    }

    /**
     * 
     * 
     * @param index
     * @param vGamma
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addGamma(final int index, final double vGamma) throws java.lang.IndexOutOfBoundsException {
        this._gammaList.add(index, new java.lang.Double(vGamma));
    }

    /**
     * 
     * 
     * @param vSpeed
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addSpeed(final double vSpeed) throws java.lang.IndexOutOfBoundsException {
        this._speedList.add(new java.lang.Double(vSpeed));
    }

    /**
     * 
     * 
     * @param index
     * @param vSpeed
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addSpeed(final int index, final double vSpeed) throws java.lang.IndexOutOfBoundsException {
        this._speedList.add(index, new java.lang.Double(vSpeed));
    }

    /**
     */
    public void deleteRho() {
        this._has_rho = false;
    }

    /**
     */
    public void deleteTheta() {
        this._has_theta = false;
    }

    /**
     * Method enumerateDelta.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateDelta() {
        return java.util.Collections.enumeration(this._deltaList);
    }

    /**
     * Method enumerateGamma.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateGamma() {
        return java.util.Collections.enumeration(this._gammaList);
    }

    /**
     * Method enumerateSpeed.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateSpeed() {
        return java.util.Collections.enumeration(this._speedList);
    }

    /**
     * Method getDelta.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the double at the given index
     */
    public double getDelta(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._deltaList.size()) {
            throw new IndexOutOfBoundsException("getDelta: Index value '" + index + "' not in range [0.." +
                (this._deltaList.size() - 1) + "]");
        }

        return ((java.lang.Double) _deltaList.get(index)).doubleValue();
    }

    /**
     * Method getDelta.Returns the contents of the collection in an
     * Array.  
     * 
     * @return this collection as an Array
     */
    public double[] getDelta() {
        int size = this._deltaList.size();
        double[] array = new double[size];
        java.util.Iterator iter = _deltaList.iterator();
        for (int index = 0; index < size; index++) {
            array[index] = ((java.lang.Double) iter.next()).doubleValue();
        }
        return array;
    }

    /**
     * Method getDeltaCount.
     * 
     * @return the size of this collection
     */
    public int getDeltaCount() {
        return this._deltaList.size();
    }

    /**
     * Method getGamma.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the double at the given index
     */
    public double getGamma(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._gammaList.size()) {
            throw new IndexOutOfBoundsException("getGamma: Index value '" + index + "' not in range [0.." +
                (this._gammaList.size() - 1) + "]");
        }

        return ((java.lang.Double) _gammaList.get(index)).doubleValue();
    }

    /**
     * Method getGamma.Returns the contents of the collection in an
     * Array.  
     * 
     * @return this collection as an Array
     */
    public double[] getGamma() {
        int size = this._gammaList.size();
        double[] array = new double[size];
        java.util.Iterator iter = _gammaList.iterator();
        for (int index = 0; index < size; index++) {
            array[index] = ((java.lang.Double) iter.next()).doubleValue();
        }
        return array;
    }

    /**
     * Method getGammaCount.
     * 
     * @return the size of this collection
     */
    public int getGammaCount() {
        return this._gammaList.size();
    }

    /**
     * Returns the value of field 'rho'. The field 'rho' has the
     * following description: represents the rate of change between
     * an option portfolio's value and the interest rate, or
     * sensitivity to the interest rate.
     *  
     * 
     * @return the value of field 'Rho'.
     */
    public double getRho() {
        return this._rho;
    }

    /**
     * Method getSpeed.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the double at the given index
     */
    public double getSpeed(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._speedList.size()) {
            throw new IndexOutOfBoundsException("getSpeed: Index value '" + index + "' not in range [0.." +
                (this._speedList.size() - 1) + "]");
        }

        return ((java.lang.Double) _speedList.get(index)).doubleValue();
    }

    /**
     * Method getSpeed.Returns the contents of the collection in an
     * Array.  
     * 
     * @return this collection as an Array
     */
    public double[] getSpeed() {
        int size = this._speedList.size();
        double[] array = new double[size];
        java.util.Iterator iter = _speedList.iterator();
        for (int index = 0; index < size; index++) {
            array[index] = ((java.lang.Double) iter.next()).doubleValue();
        }
        return array;
    }

    /**
     * Method getSpeedCount.
     * 
     * @return the size of this collection
     */
    public int getSpeedCount() {
        return this._speedList.size();
    }

    /**
     * Returns the value of field 'theta'. The field 'theta' has
     * the following description: represents the rate of change
     * between an option portfolio and time, or time sensitivity.
     *  
     * 
     * @return the value of field 'Theta'.
     */
    public double getTheta() {
        return this._theta;
    }

    /**
     * Method hasRho.
     * 
     * @return true if at least one Rho has been added
     */
    public boolean hasRho() {
        return this._has_rho;
    }

    /**
     * Method hasTheta.
     * 
     * @return true if at least one Theta has been added
     */
    public boolean hasTheta() {
        return this._has_theta;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * Method iterateDelta.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateDelta() {
        return this._deltaList.iterator();
    }

    /**
     * Method iterateGamma.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateGamma() {
        return this._gammaList.iterator();
    }

    /**
     * Method iterateSpeed.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateSpeed() {
        return this._speedList.iterator();
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     */
    public void removeAllDelta() {
        this._deltaList.clear();
    }

    /**
     */
    public void removeAllGamma() {
        this._gammaList.clear();
    }

    /**
     */
    public void removeAllSpeed() {
        this._speedList.clear();
    }

    /**
     * Method removeDelta.
     * 
     * @param vDelta
     * @return true if the object was removed from the collection.
     */
    public boolean removeDelta(final double vDelta) {
        boolean removed = _deltaList.remove(new java.lang.Double(vDelta));
        return removed;
    }

    /**
     * Method removeDeltaAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public double removeDeltaAt(final int index) {
        java.lang.Object obj = this._deltaList.remove(index);
        return ((java.lang.Double) obj).doubleValue();
    }

    /**
     * Method removeGamma.
     * 
     * @param vGamma
     * @return true if the object was removed from the collection.
     */
    public boolean removeGamma(final double vGamma) {
        boolean removed = _gammaList.remove(new java.lang.Double(vGamma));
        return removed;
    }

    /**
     * Method removeGammaAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public double removeGammaAt(final int index) {
        java.lang.Object obj = this._gammaList.remove(index);
        return ((java.lang.Double) obj).doubleValue();
    }

    /**
     * Method removeSpeed.
     * 
     * @param vSpeed
     * @return true if the object was removed from the collection.
     */
    public boolean removeSpeed(final double vSpeed) {
        boolean removed = _speedList.remove(new java.lang.Double(vSpeed));
        return removed;
    }

    /**
     * Method removeSpeedAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public double removeSpeedAt(final int index) {
        java.lang.Object obj = this._speedList.remove(index);
        return ((java.lang.Double) obj).doubleValue();
    }

    /**
     * 
     * 
     * @param index
     * @param vDelta
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setDelta(final int index, final double vDelta) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._deltaList.size()) {
            throw new IndexOutOfBoundsException("setDelta: Index value '" + index + "' not in range [0.." +
                (this._deltaList.size() - 1) + "]");
        }

        this._deltaList.set(index, new java.lang.Double(vDelta));
    }

    /**
     * 
     * 
     * @param vDeltaArray
     */
    public void setDelta(final double[] vDeltaArray) {
        //-- copy array
        _deltaList.clear();

        for (int i = 0; i < vDeltaArray.length; i++) {
            this._deltaList.add(new java.lang.Double(vDeltaArray[i]));
        }
    }

    /**
     * 
     * 
     * @param index
     * @param vGamma
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setGamma(final int index, final double vGamma) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._gammaList.size()) {
            throw new IndexOutOfBoundsException("setGamma: Index value '" + index + "' not in range [0.." +
                (this._gammaList.size() - 1) + "]");
        }

        this._gammaList.set(index, new java.lang.Double(vGamma));
    }

    /**
     * 
     * 
     * @param vGammaArray
     */
    public void setGamma(final double[] vGammaArray) {
        //-- copy array
        _gammaList.clear();

        for (int i = 0; i < vGammaArray.length; i++) {
            this._gammaList.add(new java.lang.Double(vGammaArray[i]));
        }
    }

    /**
     * Sets the value of field 'rho'. The field 'rho' has the
     * following description: represents the rate of change between
     * an option portfolio's value and the interest rate, or
     * sensitivity to the interest rate.
     *  
     * 
     * @param rho the value of field 'rho'.
     */
    public void setRho(final double rho) {
        this._rho = rho;
        this._has_rho = true;
    }

    /**
     * 
     * 
     * @param index
     * @param vSpeed
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setSpeed(final int index, final double vSpeed) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._speedList.size()) {
            throw new IndexOutOfBoundsException("setSpeed: Index value '" + index + "' not in range [0.." +
                (this._speedList.size() - 1) + "]");
        }

        this._speedList.set(index, new java.lang.Double(vSpeed));
    }

    /**
     * 
     * 
     * @param vSpeedArray
     */
    public void setSpeed(final double[] vSpeedArray) {
        //-- copy array
        _speedList.clear();

        for (int i = 0; i < vSpeedArray.length; i++) {
            this._speedList.add(new java.lang.Double(vSpeedArray[i]));
        }
    }

    /**
     * Sets the value of field 'theta'. The field 'theta' has the
     * following description: represents the rate of change between
     * an option portfolio and time, or time sensitivity.
     *  
     * 
     * @param theta the value of field 'theta'.
     */
    public void setTheta(final double theta) {
        this._theta = theta;
        this._has_theta = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.Greeks
     */
    public static plugtest.data.Greeks unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.Greeks) Unmarshaller.unmarshal(plugtest.data.Greeks.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
