/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * Holds the computational metrics such as computational time
 * required for computing any element.
 *  
 * 
 * @version $Revision$ $Date$
 */
public class Metrics implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * The time when the computation was started.
     *  
     */
    private long _startTime;

    /**
     * keeps track of state for field: _startTime
     */
    private boolean _has_startTime;

    /**
     * The time at which the computation was finished. 
     *  
     */
    private long _stopTime;

    /**
     * keeps track of state for field: _stopTime
     */
    private boolean _has_stopTime;

    /**
     * The wall time must be provided in seconds. In case of a tie
     * between same wall times,
     *  the start time and stop time attributes will be considered
     * for further accurate time calculations. 
     */
    private long _wallTime;

    /**
     * keeps track of state for field: _wallTime
     */
    private boolean _has_wallTime;

    /**
     * The time in "java.util.date" format at which the computation
     * was started.
     *  
     */
    private org.exolab.castor.types.Date _startTimeH;

    /**
     * The time in "java.util.date" format at which the computation
     * was started.
     */
    private org.exolab.castor.types.Date _stopTimeH;

    //----------------/
    //- Constructors -/
    //----------------/

    public Metrics() {
        super();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     */
    public void deleteStartTime() {
        this._has_startTime = false;
    }

    /**
     */
    public void deleteStopTime() {
        this._has_stopTime = false;
    }

    /**
     */
    public void deleteWallTime() {
        this._has_wallTime = false;
    }

    /**
     * Returns the value of field 'startTime'. The field
     * 'startTime' has the following description: The time when the
     * computation was started.
     *  
     * 
     * @return the value of field 'StartTime'.
     */
    public long getStartTime() {
        return this._startTime;
    }

    /**
     * Returns the value of field 'startTimeH'. The field
     * 'startTimeH' has the following description: The time in
     * "java.util.date" format at which the computation was
     * started.
     *  
     * 
     * @return the value of field 'StartTimeH'.
     */
    public org.exolab.castor.types.Date getStartTimeH() {
        return this._startTimeH;
    }

    /**
     * Returns the value of field 'stopTime'. The field 'stopTime'
     * has the following description: The time at which the
     * computation was finished. 
     *  
     * 
     * @return the value of field 'StopTime'.
     */
    public long getStopTime() {
        return this._stopTime;
    }

    /**
     * Returns the value of field 'stopTimeH'. The field
     * 'stopTimeH' has the following description: The time in
     * "java.util.date" format at which the computation was
     * started.
     * 
     * @return the value of field 'StopTimeH'.
     */
    public org.exolab.castor.types.Date getStopTimeH() {
        return this._stopTimeH;
    }

    /**
     * Returns the value of field 'wallTime'. The field 'wallTime'
     * has the following description: The wall time must be
     * provided in seconds. In case of a tie between same wall
     * times,
     *  the start time and stop time attributes will be considered
     * for further accurate time calculations. 
     * 
     * @return the value of field 'WallTime'.
     */
    public long getWallTime() {
        return this._wallTime;
    }

    /**
     * Method hasStartTime.
     * 
     * @return true if at least one StartTime has been added
     */
    public boolean hasStartTime() {
        return this._has_startTime;
    }

    /**
     * Method hasStopTime.
     * 
     * @return true if at least one StopTime has been added
     */
    public boolean hasStopTime() {
        return this._has_stopTime;
    }

    /**
     * Method hasWallTime.
     * 
     * @return true if at least one WallTime has been added
     */
    public boolean hasWallTime() {
        return this._has_wallTime;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     * Sets the value of field 'startTime'. The field 'startTime'
     * has the following description: The time when the computation
     * was started.
     *  
     * 
     * @param startTime the value of field 'startTime'.
     */
    public void setStartTime(final long startTime) {
        this._startTime = startTime;
        this._has_startTime = true;
    }

    /**
     * Sets the value of field 'startTimeH'. The field 'startTimeH'
     * has the following description: The time in "java.util.date"
     * format at which the computation was started.
     *  
     * 
     * @param startTimeH the value of field 'startTimeH'.
     */
    public void setStartTimeH(final org.exolab.castor.types.Date startTimeH) {
        this._startTimeH = startTimeH;
    }

    /**
     * Sets the value of field 'stopTime'. The field 'stopTime' has
     * the following description: The time at which the computation
     * was finished. 
     *  
     * 
     * @param stopTime the value of field 'stopTime'.
     */
    public void setStopTime(final long stopTime) {
        this._stopTime = stopTime;
        this._has_stopTime = true;
    }

    /**
     * Sets the value of field 'stopTimeH'. The field 'stopTimeH'
     * has the following description: The time in "java.util.date"
     * format at which the computation was started.
     * 
     * @param stopTimeH the value of field 'stopTimeH'.
     */
    public void setStopTimeH(final org.exolab.castor.types.Date stopTimeH) {
        this._stopTimeH = stopTimeH;
    }

    /**
     * Sets the value of field 'wallTime'. The field 'wallTime' has
     * the following description: The wall time must be provided in
     * seconds. In case of a tie between same wall times,
     *  the start time and stop time attributes will be considered
     * for further accurate time calculations. 
     * 
     * @param wallTime the value of field 'wallTime'.
     */
    public void setWallTime(final long wallTime) {
        this._wallTime = wallTime;
        this._has_wallTime = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.Metrics
     */
    public static plugtest.data.Metrics unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.Metrics) Unmarshaller.unmarshal(plugtest.data.Metrics.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
