/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * Holds computed the price/premium of the basket option. 
 *  
 * 
 * @version $Revision$ $Date$
 */
public class Price implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * The amount the purchaser of a call option must pay to the
     *  writer. (Investopedia) 
     */
    private double _callPrice;

    /**
     * keeps track of state for field: _callPrice
     */
    private boolean _has_callPrice;

    /**
     * The amount the purchaser of a put option must pay to the
     *  writer. (Investopedia) 
     */
    private double _putPrice;

    /**
     * keeps track of state for field: _putPrice
     */
    private boolean _has_putPrice;

    /**
     * is the averaging the squared distance of any random
     * variable's possible values from the expected value (mean)
     *  
     */
    private double _variance;

    /**
     * keeps track of state for field: _variance
     */
    private boolean _has_variance;

    /**
     * the degree of assurance that a specified failure rate is not
     * exceeded. 
     *  
     */
    private double _CI;

    /**
     * keeps track of state for field: _CI
     */
    private boolean _has_CI;

    //----------------/
    //- Constructors -/
    //----------------/

    public Price() {
        super();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     */
    public void deleteCI() {
        this._has_CI = false;
    }

    /**
     */
    public void deleteCallPrice() {
        this._has_callPrice = false;
    }

    /**
     */
    public void deletePutPrice() {
        this._has_putPrice = false;
    }

    /**
     */
    public void deleteVariance() {
        this._has_variance = false;
    }

    /**
     * Returns the value of field 'CI'. The field 'CI' has the
     * following description: the degree of assurance that a
     * specified failure rate is not exceeded. 
     *  
     * 
     * @return the value of field 'CI'.
     */
    public double getCI() {
        return this._CI;
    }

    /**
     * Returns the value of field 'callPrice'. The field
     * 'callPrice' has the following description: The amount the
     * purchaser of a call option must pay to the
     *  writer. (Investopedia) 
     * 
     * @return the value of field 'CallPrice'.
     */
    public double getCallPrice() {
        return this._callPrice;
    }

    /**
     * Returns the value of field 'putPrice'. The field 'putPrice'
     * has the following description: The amount the purchaser of a
     * put option must pay to the
     *  writer. (Investopedia) 
     * 
     * @return the value of field 'PutPrice'.
     */
    public double getPutPrice() {
        return this._putPrice;
    }

    /**
     * Returns the value of field 'variance'. The field 'variance'
     * has the following description: is the averaging the squared
     * distance of any random variable's possible values from the
     * expected value (mean)
     *  
     * 
     * @return the value of field 'Variance'.
     */
    public double getVariance() {
        return this._variance;
    }

    /**
     * Method hasCI.
     * 
     * @return true if at least one CI has been added
     */
    public boolean hasCI() {
        return this._has_CI;
    }

    /**
     * Method hasCallPrice.
     * 
     * @return true if at least one CallPrice has been added
     */
    public boolean hasCallPrice() {
        return this._has_callPrice;
    }

    /**
     * Method hasPutPrice.
     * 
     * @return true if at least one PutPrice has been added
     */
    public boolean hasPutPrice() {
        return this._has_putPrice;
    }

    /**
     * Method hasVariance.
     * 
     * @return true if at least one Variance has been added
     */
    public boolean hasVariance() {
        return this._has_variance;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     * Sets the value of field 'CI'. The field 'CI' has the
     * following description: the degree of assurance that a
     * specified failure rate is not exceeded. 
     *  
     * 
     * @param CI the value of field 'CI'.
     */
    public void setCI(final double CI) {
        this._CI = CI;
        this._has_CI = true;
    }

    /**
     * Sets the value of field 'callPrice'. The field 'callPrice'
     * has the following description: The amount the purchaser of a
     * call option must pay to the
     *  writer. (Investopedia) 
     * 
     * @param callPrice the value of field 'callPrice'.
     */
    public void setCallPrice(final double callPrice) {
        this._callPrice = callPrice;
        this._has_callPrice = true;
    }

    /**
     * Sets the value of field 'putPrice'. The field 'putPrice' has
     * the following description: The amount the purchaser of a put
     * option must pay to the
     *  writer. (Investopedia) 
     * 
     * @param putPrice the value of field 'putPrice'.
     */
    public void setPutPrice(final double putPrice) {
        this._putPrice = putPrice;
        this._has_putPrice = true;
    }

    /**
     * Sets the value of field 'variance'. The field 'variance' has
     * the following description: is the averaging the squared
     * distance of any random variable's possible values from the
     * expected value (mean)
     *  
     * 
     * @param variance the value of field 'variance'.
     */
    public void setVariance(final double variance) {
        this._variance = variance;
        this._has_variance = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.Price
     */
    public static plugtest.data.Price unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.Price) Unmarshaller.unmarshal(plugtest.data.Price.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
