/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * Holds the computed prices, greeks and relevent computational
 * parameters. 
 *  
 * 
 * @version $Revision$ $Date$
 */
public class Result implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * It is the test case id for which this result stands for.
     */
    private int _testCaseID;

    /**
     * keeps track of state for field: _testCaseID
     */
    private boolean _has_testCaseID;

    /**
     * Holds computed the price/premium of the basket option. 
     *  
     */
    private plugtest.data.Price _price;

    /**
     * In mathematical finance, the Greeks are the quantities
     * representing
     *  the market sensitivities of derivatives such as options.
     * Each "Greek" measures a
     *  different aspect of the risk in an option position, and
     * corresponds to a parameter
     *  on which the value of an instrument or portfolio of
     * financial instruments is
     *  dependent. The name is used because the parameters are
     * often denoted by Greek
     *  letters. (Wikipedia) 
     */
    private plugtest.data.Greeks _greeks;

    /**
     * Holds the computational metrics such as computational time
     * required for computing any element.
     *  
     */
    private plugtest.data.Metrics _metrics;

    //----------------/
    //- Constructors -/
    //----------------/

    public Result() {
        super();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     */
    public void deleteTestCaseID() {
        this._has_testCaseID = false;
    }

    /**
     * Returns the value of field 'greeks'. The field 'greeks' has
     * the following description: In mathematical finance, the
     * Greeks are the quantities representing
     *  the market sensitivities of derivatives such as options.
     * Each "Greek" measures a
     *  different aspect of the risk in an option position, and
     * corresponds to a parameter
     *  on which the value of an instrument or portfolio of
     * financial instruments is
     *  dependent. The name is used because the parameters are
     * often denoted by Greek
     *  letters. (Wikipedia) 
     * 
     * @return the value of field 'Greeks'.
     */
    public plugtest.data.Greeks getGreeks() {
        return this._greeks;
    }

    /**
     * Returns the value of field 'metrics'. The field 'metrics'
     * has the following description: Holds the computational
     * metrics such as computational time required for computing
     * any element.
     *  
     * 
     * @return the value of field 'Metrics'.
     */
    public plugtest.data.Metrics getMetrics() {
        return this._metrics;
    }

    /**
     * Returns the value of field 'price'. The field 'price' has
     * the following description: Holds computed the price/premium
     * of the basket option. 
     *  
     * 
     * @return the value of field 'Price'.
     */
    public plugtest.data.Price getPrice() {
        return this._price;
    }

    /**
     * Returns the value of field 'testCaseID'. The field
     * 'testCaseID' has the following description: It is the test
     * case id for which this result stands for.
     * 
     * @return the value of field 'TestCaseID'.
     */
    public int getTestCaseID() {
        return this._testCaseID;
    }

    /**
     * Method hasTestCaseID.
     * 
     * @return true if at least one TestCaseID has been added
     */
    public boolean hasTestCaseID() {
        return this._has_testCaseID;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     * Sets the value of field 'greeks'. The field 'greeks' has the
     * following description: In mathematical finance, the Greeks
     * are the quantities representing
     *  the market sensitivities of derivatives such as options.
     * Each "Greek" measures a
     *  different aspect of the risk in an option position, and
     * corresponds to a parameter
     *  on which the value of an instrument or portfolio of
     * financial instruments is
     *  dependent. The name is used because the parameters are
     * often denoted by Greek
     *  letters. (Wikipedia) 
     * 
     * @param greeks the value of field 'greeks'.
     */
    public void setGreeks(final plugtest.data.Greeks greeks) {
        this._greeks = greeks;
    }

    /**
     * Sets the value of field 'metrics'. The field 'metrics' has
     * the following description: Holds the computational metrics
     * such as computational time required for computing any
     * element.
     *  
     * 
     * @param metrics the value of field 'metrics'.
     */
    public void setMetrics(final plugtest.data.Metrics metrics) {
        this._metrics = metrics;
    }

    /**
     * Sets the value of field 'price'. The field 'price' has the
     * following description: Holds computed the price/premium of
     * the basket option. 
     *  
     * 
     * @param price the value of field 'price'.
     */
    public void setPrice(final plugtest.data.Price price) {
        this._price = price;
    }

    /**
     * Sets the value of field 'testCaseID'. The field 'testCaseID'
     * has the following description: It is the test case id for
     * which this result stands for.
     * 
     * @param testCaseID the value of field 'testCaseID'.
     */
    public void setTestCaseID(final int testCaseID) {
        this._testCaseID = testCaseID;
        this._has_testCaseID = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.Result
     */
    public static plugtest.data.Result unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.Result) Unmarshaller.unmarshal(plugtest.data.Result.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
