/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * It describes an individual test case for the challenge. For
 * further description, please check the documentation of the
 * schema in the compile/config directory of this archive. 
 *  
 * 
 * @version $Revision$ $Date$
 */
public class TestCase implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * Each test case has a unique identifier. The results computed
     * for a perticular test case should include the relevant
     * identifier.
     *  
     */
    private int _testCaseID;

    /**
     * keeps track of state for field: _testCaseID
     */
    private boolean _has_testCaseID;

    /**
     * The basket option consists of a subset of asset pool that
     * needs to be priced. The basket contains the indices of the
     * assets from the
     *  assetpool. The indices of the assets from the assetpool are
     * 0-based. For instace basket=[0 11 90] means that the basket
     * for the current testcase includes the 0th, 11th and 90th
     * asset from the asset pool.
     *  
     */
    private java.util.List _basketList;

    /**
     * The type of the option that the basket represents. (whether
     * call or put) 
     *  
     */
    private plugtest.data.types.OptionType _option;

    /**
     * The payoff of the option that needs to be computed.
     *  
     */
    private plugtest.data.types.PayOffType _payOff;

    /**
     * The type of the option contract, whether it is a vanilla
     * option or a barrier option. 
     *  
     */
    private plugtest.data.types.AlgorithmType _algorithm;

    /**
     * The time remaining until a call or put option expires. 
     *  
     */
    private double _timeToMaturity;

    /**
     * keeps track of state for field: _timeToMaturity
     */
    private boolean _has_timeToMaturity;

    /**
     * the barrier level in percentage. For barrier options the
     * strike price usually depends on the spot price of the
     * underlying basket. It is usually +/-1% of the spot price of
     * the basket.
     *  
     */
    private double _barrier = 0.0;

    /**
     * keeps track of state for field: _barrier
     */
    private boolean _has_barrier;

    //----------------/
    //- Constructors -/
    //----------------/

    public TestCase() {
        super();
        this._basketList = new java.util.ArrayList();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * 
     * 
     * @param vBasket
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addBasket(final int vBasket) throws java.lang.IndexOutOfBoundsException {
        this._basketList.add(new java.lang.Integer(vBasket));
    }

    /**
     * 
     * 
     * @param index
     * @param vBasket
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addBasket(final int index, final int vBasket) throws java.lang.IndexOutOfBoundsException {
        this._basketList.add(index, new java.lang.Integer(vBasket));
    }

    /**
     */
    public void deleteBarrier() {
        this._has_barrier = false;
    }

    /**
     */
    public void deleteTestCaseID() {
        this._has_testCaseID = false;
    }

    /**
     */
    public void deleteTimeToMaturity() {
        this._has_timeToMaturity = false;
    }

    /**
     * Method enumerateBasket.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateBasket() {
        return java.util.Collections.enumeration(this._basketList);
    }

    /**
     * Returns the value of field 'algorithm'. The field
     * 'algorithm' has the following description: The type of the
     * option contract, whether it is a vanilla option or a barrier
     * option. 
     *  
     * 
     * @return the value of field 'Algorithm'.
     */
    public plugtest.data.types.AlgorithmType getAlgorithm() {
        return this._algorithm;
    }

    /**
     * Returns the value of field 'barrier'. The field 'barrier'
     * has the following description: the barrier level in
     * percentage. For barrier options the strike price usually
     * depends on the spot price of the underlying basket. It is
     * usually +/-1% of the spot price of the basket.
     *  
     * 
     * @return the value of field 'Barrier'.
     */
    public double getBarrier() {
        return this._barrier;
    }

    /**
     * Method getBasket.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the int at the given index
     */
    public int getBasket(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._basketList.size()) {
            throw new IndexOutOfBoundsException("getBasket: Index value '" + index + "' not in range [0.." +
                (this._basketList.size() - 1) + "]");
        }

        return ((java.lang.Integer) _basketList.get(index)).intValue();
    }

    /**
     * Method getBasket.Returns the contents of the collection in
     * an Array.  
     * 
     * @return this collection as an Array
     */
    public int[] getBasket() {
        int size = this._basketList.size();
        int[] array = new int[size];
        java.util.Iterator iter = _basketList.iterator();
        for (int index = 0; index < size; index++) {
            array[index] = ((java.lang.Integer) iter.next()).intValue();
        }
        return array;
    }

    /**
     * Method getBasketCount.
     * 
     * @return the size of this collection
     */
    public int getBasketCount() {
        return this._basketList.size();
    }

    /**
     * Returns the value of field 'option'. The field 'option' has
     * the following description: The type of the option that the
     * basket represents. (whether call or put) 
     *  
     * 
     * @return the value of field 'Option'.
     */
    public plugtest.data.types.OptionType getOption() {
        return this._option;
    }

    /**
     * Returns the value of field 'payOff'. The field 'payOff' has
     * the following description: The payoff of the option that
     * needs to be computed.
     *  
     * 
     * @return the value of field 'PayOff'.
     */
    public plugtest.data.types.PayOffType getPayOff() {
        return this._payOff;
    }

    /**
     * Returns the value of field 'testCaseID'. The field
     * 'testCaseID' has the following description: Each test case
     * has a unique identifier. The results computed for a
     * perticular test case should include the relevant identifier.
     *  
     * 
     * @return the value of field 'TestCaseID'.
     */
    public int getTestCaseID() {
        return this._testCaseID;
    }

    /**
     * Returns the value of field 'timeToMaturity'. The field
     * 'timeToMaturity' has the following description: The time
     * remaining until a call or put option expires. 
     *  
     * 
     * @return the value of field 'TimeToMaturity'.
     */
    public double getTimeToMaturity() {
        return this._timeToMaturity;
    }

    /**
     * Method hasBarrier.
     * 
     * @return true if at least one Barrier has been added
     */
    public boolean hasBarrier() {
        return this._has_barrier;
    }

    /**
     * Method hasTestCaseID.
     * 
     * @return true if at least one TestCaseID has been added
     */
    public boolean hasTestCaseID() {
        return this._has_testCaseID;
    }

    /**
     * Method hasTimeToMaturity.
     * 
     * @return true if at least one TimeToMaturity has been added
     */
    public boolean hasTimeToMaturity() {
        return this._has_timeToMaturity;
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * Method iterateBasket.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateBasket() {
        return this._basketList.iterator();
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     */
    public void removeAllBasket() {
        this._basketList.clear();
    }

    /**
     * Method removeBasket.
     * 
     * @param vBasket
     * @return true if the object was removed from the collection.
     */
    public boolean removeBasket(final int vBasket) {
        boolean removed = _basketList.remove(new java.lang.Integer(vBasket));
        return removed;
    }

    /**
     * Method removeBasketAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public int removeBasketAt(final int index) {
        java.lang.Object obj = this._basketList.remove(index);
        return ((java.lang.Integer) obj).intValue();
    }

    /**
     * Sets the value of field 'algorithm'. The field 'algorithm'
     * has the following description: The type of the option
     * contract, whether it is a vanilla option or a barrier
     * option. 
     *  
     * 
     * @param algorithm the value of field 'algorithm'.
     */
    public void setAlgorithm(final plugtest.data.types.AlgorithmType algorithm) {
        this._algorithm = algorithm;
    }

    /**
     * Sets the value of field 'barrier'. The field 'barrier' has
     * the following description: the barrier level in percentage.
     * For barrier options the strike price usually depends on the
     * spot price of the underlying basket. It is usually +/-1% of
     * the spot price of the basket.
     *  
     * 
     * @param barrier the value of field 'barrier'.
     */
    public void setBarrier(final double barrier) {
        this._barrier = barrier;
        this._has_barrier = true;
    }

    /**
     * 
     * 
     * @param index
     * @param vBasket
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setBasket(final int index, final int vBasket) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._basketList.size()) {
            throw new IndexOutOfBoundsException("setBasket: Index value '" + index + "' not in range [0.." +
                (this._basketList.size() - 1) + "]");
        }

        this._basketList.set(index, new java.lang.Integer(vBasket));
    }

    /**
     * 
     * 
     * @param vBasketArray
     */
    public void setBasket(final int[] vBasketArray) {
        //-- copy array
        _basketList.clear();

        for (int i = 0; i < vBasketArray.length; i++) {
            this._basketList.add(new java.lang.Integer(vBasketArray[i]));
        }
    }

    /**
     * Sets the value of field 'option'. The field 'option' has the
     * following description: The type of the option that the
     * basket represents. (whether call or put) 
     *  
     * 
     * @param option the value of field 'option'.
     */
    public void setOption(final plugtest.data.types.OptionType option) {
        this._option = option;
    }

    /**
     * Sets the value of field 'payOff'. The field 'payOff' has the
     * following description: The payoff of the option that needs
     * to be computed.
     *  
     * 
     * @param payOff the value of field 'payOff'.
     */
    public void setPayOff(final plugtest.data.types.PayOffType payOff) {
        this._payOff = payOff;
    }

    /**
     * Sets the value of field 'testCaseID'. The field 'testCaseID'
     * has the following description: Each test case has a unique
     * identifier. The results computed for a perticular test case
     * should include the relevant identifier.
     *  
     * 
     * @param testCaseID the value of field 'testCaseID'.
     */
    public void setTestCaseID(final int testCaseID) {
        this._testCaseID = testCaseID;
        this._has_testCaseID = true;
    }

    /**
     * Sets the value of field 'timeToMaturity'. The field
     * 'timeToMaturity' has the following description: The time
     * remaining until a call or put option expires. 
     *  
     * 
     * @param timeToMaturity the value of field 'timeToMaturity'.
     */
    public void setTimeToMaturity(final double timeToMaturity) {
        this._timeToMaturity = timeToMaturity;
        this._has_timeToMaturity = true;
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.TestCase
     */
    public static plugtest.data.TestCase unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.TestCase) Unmarshaller.unmarshal(plugtest.data.TestCase.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
