/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;


/**
 * Testcases defines a collection of testcases that will be
 * provided to the participants. 
 * 
 * @version $Revision$ $Date$
 */
public class TestCases implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * These are the constants that will be used for computing
     * prices/greeks of all the test cases 
     */
    private plugtest.data.BasketParams _basketParams;

    /**
     * It describes an individual test case for the challenge. For
     * further description, please check the documentation of the
     * schema in the compile/config directory of this archive. 
     *  
     */
    private java.util.List _testCaseList;

    //----------------/
    //- Constructors -/
    //----------------/

    public TestCases() {
        super();
        this._testCaseList = new java.util.ArrayList();
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * 
     * 
     * @param vTestCase
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addTestCase(final plugtest.data.TestCase vTestCase)
            throws java.lang.IndexOutOfBoundsException {
        this._testCaseList.add(vTestCase);
    }

    /**
     * 
     * 
     * @param index
     * @param vTestCase
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void addTestCase(final int index, final plugtest.data.TestCase vTestCase)
            throws java.lang.IndexOutOfBoundsException {
        this._testCaseList.add(index, vTestCase);
    }

    /**
     * Method enumerateTestCase.
     * 
     * @return an Enumeration over all possible elements of this
     * collection
     */
    public java.util.Enumeration enumerateTestCase() {
        return java.util.Collections.enumeration(this._testCaseList);
    }

    /**
     * Returns the value of field 'basketParams'. The field
     * 'basketParams' has the following description: These are the
     * constants that will be used for computing prices/greeks of
     * all the test cases 
     * 
     * @return the value of field 'BasketParams'.
     */
    public plugtest.data.BasketParams getBasketParams() {
        return this._basketParams;
    }

    /**
     * Method getTestCase.
     * 
     * @param index
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     * @return the value of the plugtest.data.TestCase at the given
     * index
     */
    public plugtest.data.TestCase getTestCase(final int index) throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._testCaseList.size()) {
            throw new IndexOutOfBoundsException("getTestCase: Index value '" + index + "' not in range [0.." +
                (this._testCaseList.size() - 1) + "]");
        }

        return (plugtest.data.TestCase) _testCaseList.get(index);
    }

    /**
     * Method getTestCase.Returns the contents of the collection in
     * an Array.  <p>Note:  Just in case the collection contents
     * are changing in another thread, we pass a 0-length Array of
     * the correct type into the API call.  This way we <i>know</i>
     * that the Array returned is of exactly the correct length.
     * 
     * @return this collection as an Array
     */
    public plugtest.data.TestCase[] getTestCase() {
        plugtest.data.TestCase[] array = new plugtest.data.TestCase[0];
        return (plugtest.data.TestCase[]) this._testCaseList.toArray(array);
    }

    /**
     * Method getTestCaseCount.
     * 
     * @return the size of this collection
     */
    public int getTestCaseCount() {
        return this._testCaseList.size();
    }

    /**
     * Method isValid.
     * 
     * @return true if this object is valid according to the schema
     */
    public boolean isValid() {
        try {
            validate();
        } catch (org.exolab.castor.xml.ValidationException vex) {
            return false;
        }
        return true;
    }

    /**
     * Method iterateTestCase.
     * 
     * @return an Iterator over all possible elements in this
     * collection
     */
    public java.util.Iterator iterateTestCase() {
        return this._testCaseList.iterator();
    }

    /**
     * 
     * 
     * @param out
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void marshal(final java.io.Writer out) throws org.exolab.castor.xml.MarshalException,
            org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, out);
    }

    /**
     * 
     * 
     * @param handler
     * @throws java.io.IOException if an IOException occurs during
     * marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     */
    public void marshal(final org.xml.sax.ContentHandler handler) throws java.io.IOException,
            org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        Marshaller.marshal(this, handler);
    }

    /**
     */
    public void removeAllTestCase() {
        this._testCaseList.clear();
    }

    /**
     * Method removeTestCase.
     * 
     * @param vTestCase
     * @return true if the object was removed from the collection.
     */
    public boolean removeTestCase(final plugtest.data.TestCase vTestCase) {
        boolean removed = _testCaseList.remove(vTestCase);
        return removed;
    }

    /**
     * Method removeTestCaseAt.
     * 
     * @param index
     * @return the element removed from the collection
     */
    public plugtest.data.TestCase removeTestCaseAt(final int index) {
        java.lang.Object obj = this._testCaseList.remove(index);
        return (plugtest.data.TestCase) obj;
    }

    /**
     * Sets the value of field 'basketParams'. The field
     * 'basketParams' has the following description: These are the
     * constants that will be used for computing prices/greeks of
     * all the test cases 
     * 
     * @param basketParams the value of field 'basketParams'.
     */
    public void setBasketParams(final plugtest.data.BasketParams basketParams) {
        this._basketParams = basketParams;
    }

    /**
     * 
     * 
     * @param index
     * @param vTestCase
     * @throws java.lang.IndexOutOfBoundsException if the index
     * given is outside the bounds of the collection
     */
    public void setTestCase(final int index, final plugtest.data.TestCase vTestCase)
            throws java.lang.IndexOutOfBoundsException {
        // check bounds for index
        if (index < 0 || index >= this._testCaseList.size()) {
            throw new IndexOutOfBoundsException("setTestCase: Index value '" + index + "' not in range [0.." +
                (this._testCaseList.size() - 1) + "]");
        }

        this._testCaseList.set(index, vTestCase);
    }

    /**
     * 
     * 
     * @param vTestCaseArray
     */
    public void setTestCase(final plugtest.data.TestCase[] vTestCaseArray) {
        //-- copy array
        _testCaseList.clear();

        for (int i = 0; i < vTestCaseArray.length; i++) {
            this._testCaseList.add(vTestCaseArray[i]);
        }
    }

    /**
     * Method unmarshal.
     * 
     * @param reader
     * @throws org.exolab.castor.xml.MarshalException if object is
     * null or if any SAXException is thrown during marshaling
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     * @return the unmarshaled plugtest.data.TestCases
     */
    public static plugtest.data.TestCases unmarshal(final java.io.Reader reader)
            throws org.exolab.castor.xml.MarshalException, org.exolab.castor.xml.ValidationException {
        return (plugtest.data.TestCases) Unmarshaller.unmarshal(plugtest.data.TestCases.class, reader);
    }

    /**
     * 
     * 
     * @throws org.exolab.castor.xml.ValidationException if this
     * object is an invalid instance according to the schema
     */
    public void validate() throws org.exolab.castor.xml.ValidationException {
        org.exolab.castor.xml.Validator validator = new org.exolab.castor.xml.Validator();
        validator.validate(this);
    }

}
