/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data.types;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import java.util.Hashtable;


/**
 * Defines the type of the algorithm to use to price the
 * option/derivative product
 *  
 * 
 * @version $Revision$ $Date$
 */
public class AlgorithmType implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * The Vanilla type
     */
    public static final int VANILLA_TYPE = 0;

    /**
     * The instance of the Vanilla type
     */
    public static final AlgorithmType VANILLA = new AlgorithmType(VANILLA_TYPE, "Vanilla");

    /**
     * The BarrierDownIn type
     */
    public static final int BARRIERDOWNIN_TYPE = 1;

    /**
     * The instance of the BarrierDownIn type
     */
    public static final AlgorithmType BARRIERDOWNIN = new AlgorithmType(BARRIERDOWNIN_TYPE, "BarrierDownIn");

    /**
     * The BarrierDownOut type
     */
    public static final int BARRIERDOWNOUT_TYPE = 2;

    /**
     * The instance of the BarrierDownOut type
     */
    public static final AlgorithmType BARRIERDOWNOUT = new AlgorithmType(BARRIERDOWNOUT_TYPE,
        "BarrierDownOut");

    /**
     * The BarrierUpIn type
     */
    public static final int BARRIERUPIN_TYPE = 3;

    /**
     * The instance of the BarrierUpIn type
     */
    public static final AlgorithmType BARRIERUPIN = new AlgorithmType(BARRIERUPIN_TYPE, "BarrierUpIn");

    /**
     * The BarrierUpOut type
     */
    public static final int BARRIERUPOUT_TYPE = 4;

    /**
     * The instance of the BarrierUpOut type
     */
    public static final AlgorithmType BARRIERUPOUT = new AlgorithmType(BARRIERUPOUT_TYPE, "BarrierUpOut");

    /**
     * Field _memberTable.
     */
    private static java.util.Hashtable _memberTable = init();

    /**
     * Field type.
     */
    private int type = -1;

    /**
     * Field stringValue.
     */
    private java.lang.String stringValue = null;

    //----------------/
    //- Constructors -/
    //----------------/

    private AlgorithmType(final int type, final java.lang.String value) {
        super();
        this.type = type;
        this.stringValue = value;
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * Method enumerate.Returns an enumeration of all possible
     * instances of AlgorithmType
     * 
     * @return an Enumeration over all possible instances of
     * AlgorithmType
     */
    public static java.util.Enumeration enumerate() {
        return _memberTable.elements();
    }

    /**
     * Method getType.Returns the type of this AlgorithmType
     * 
     * @return the type of this AlgorithmType
     */
    public int getType() {
        return this.type;
    }

    /**
     * Method init.
     * 
     * @return the initialized Hashtable for the member table
     */
    private static java.util.Hashtable init() {
        Hashtable members = new Hashtable();
        members.put("Vanilla", VANILLA);
        members.put("BarrierDownIn", BARRIERDOWNIN);
        members.put("BarrierDownOut", BARRIERDOWNOUT);
        members.put("BarrierUpIn", BARRIERUPIN);
        members.put("BarrierUpOut", BARRIERUPOUT);
        return members;
    }

    /**
     * Method readResolve. will be called during deserialization to
     * replace the deserialized object with the correct constant
     * instance.
     * 
     * @return this deserialized object
     */
    private java.lang.Object readResolve() {
        return valueOf(this.stringValue);
    }

    /**
     * Method toString.Returns the String representation of this
     * AlgorithmType
     * 
     * @return the String representation of this AlgorithmType
     */
    public java.lang.String toString() {
        return this.stringValue;
    }

    /**
     * Method valueOf.Returns a new AlgorithmType based on the
     * given String value.
     * 
     * @param string
     * @return the AlgorithmType value of parameter 'string'
     */
    public static plugtest.data.types.AlgorithmType valueOf(final java.lang.String string) {
        java.lang.Object obj = null;
        if (string != null) {
            obj = _memberTable.get(string);
        }
        if (obj == null) {
            String err = "" + string + " is not a valid AlgorithmType";
            throw new IllegalArgumentException(err);
        }
        return (AlgorithmType) obj;
    }

}
