/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.data.types;

//---------------------------------/
//- Imported classes and packages -/
//---------------------------------/

import java.util.Hashtable;


/**
 * Defines type of the option, whether call or put 
 *  
 * 
 * @version $Revision$ $Date$
 */
public class OptionType implements java.io.Serializable {

    //--------------------------/
    //- Class/Member Variables -/
    //--------------------------/

    /**
     * The Call type
     */
    public static final int CALL_TYPE = 0;

    /**
     * The instance of the Call type
     */
    public static final OptionType CALL = new OptionType(CALL_TYPE, "Call");

    /**
     * The Put type
     */
    public static final int PUT_TYPE = 1;

    /**
     * The instance of the Put type
     */
    public static final OptionType PUT = new OptionType(PUT_TYPE, "Put");

    /**
     * Field _memberTable.
     */
    private static java.util.Hashtable _memberTable = init();

    /**
     * Field type.
     */
    private int type = -1;

    /**
     * Field stringValue.
     */
    private java.lang.String stringValue = null;

    //----------------/
    //- Constructors -/
    //----------------/

    private OptionType(final int type, final java.lang.String value) {
        super();
        this.type = type;
        this.stringValue = value;
    }

    //-----------/
    //- Methods -/
    //-----------/

    /**
     * Method enumerate.Returns an enumeration of all possible
     * instances of OptionType
     * 
     * @return an Enumeration over all possible instances of
     * OptionType
     */
    public static java.util.Enumeration enumerate() {
        return _memberTable.elements();
    }

    /**
     * Method getType.Returns the type of this OptionType
     * 
     * @return the type of this OptionType
     */
    public int getType() {
        return this.type;
    }

    /**
     * Method init.
     * 
     * @return the initialized Hashtable for the member table
     */
    private static java.util.Hashtable init() {
        Hashtable members = new Hashtable();
        members.put("Call", CALL);
        members.put("Put", PUT);
        return members;
    }

    /**
     * Method readResolve. will be called during deserialization to
     * replace the deserialized object with the correct constant
     * instance.
     * 
     * @return this deserialized object
     */
    private java.lang.Object readResolve() {
        return valueOf(this.stringValue);
    }

    /**
     * Method toString.Returns the String representation of this
     * OptionType
     * 
     * @return the String representation of this OptionType
     */
    public java.lang.String toString() {
        return this.stringValue;
    }

    /**
     * Method valueOf.Returns a new OptionType based on the given
     * String value.
     * 
     * @param string
     * @return the OptionType value of parameter 'string'
     */
    public static plugtest.data.types.OptionType valueOf(final java.lang.String string) {
        java.lang.Object obj = null;
        if (string != null) {
            obj = _memberTable.get(string);
        }
        if (obj == null) {
            String err = "" + string + " is not a valid OptionType";
            throw new IllegalArgumentException(err);
        }
        return (OptionType) obj;
    }

}
