/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.option;

import Jama.Matrix;
import plugtest.data.AssetPool;
import plugtest.data.BasketParams;
import plugtest.data.Result;
import plugtest.data.TestCase;
import plugtest.pricing.engines.AbstractPricingEngine;

import java.io.Serializable;


/**
 * Abstract basket option
 *
 * @author The ProActive Team
 */
public abstract class AbstractBasketOption implements Serializable {

    protected AssetPool subAssetPool;
    protected BasketParams basketParams;
    protected Matrix correlationMatrix;
    protected TestCase testCase;

    protected Result result;

    protected AbstractPricingEngine engine;

    AbstractBasketOption(BasketParams basketParams2, Matrix corMatrix, TestCase testCase,
            AssetPool subAssetPool) {
        this.basketParams = basketParams2;
        this.correlationMatrix = corMatrix;
        this.testCase = testCase;
        this.subAssetPool = subAssetPool;
    }

    public AbstractBasketOption(BasketParams basketParams2) {
        this.basketParams = basketParams2;
    }

    public AbstractPricingEngine getPricingEngine() {
        return engine;
    }

    public void setPricingEngine(final AbstractPricingEngine engine) {
        this.engine = engine;
    }

    /**
     * abstract method for computing price using monte carlo based methods
     *
     * @throws Exception
     */
    public abstract void calculateMC() throws Exception;

    public BasketParams getBasketParams() {
        return basketParams;
    }

    public void setBasketParams(BasketParams basketParams) {
        this.basketParams = basketParams;
    }

    public Matrix getCorrelationMatrix() {
        return correlationMatrix;
    }

    public void setCorrelationMatrix(Matrix correlationMatrix) {
        this.correlationMatrix = correlationMatrix;
    }

    public AbstractPricingEngine getEngine() {
        return engine;
    }

    public void setEngine(AbstractPricingEngine engine) {
        this.engine = engine;
    }

    public Result getResult() {
        return result;
    }

    public void setResult(Result result) {
        this.result = result;
    }

}
