/*
 * ################################################################
 *
 * ProActive: The Java(TM) library for Parallel, Distributed,
 *            Concurrent computing with Security and Mobility
 *
 * Copyright (C) 1997-2008 INRIA/University of Nice-Sophia Antipolis
 * Contact: proactive@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Initial developer(s):               The ProActive Team
 *                        http://proactive.inria.fr/team_members.htm
 *  Contributor(s):
 *
 * ################################################################
 * $$PROACTIVE_INITIAL_DEV$$
 */
package plugtest.pricing.util;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import plugtest.data.*;

import java.io.OutputStream;
import java.io.PrintWriter;


/**
 * Prints the given data onto the given or standard output stream
 *
 * @author The ProActive Team
 */
public class DisplayData {

    Result result;

    DisplayData(Result rs) {
        this.result = rs;
    }

    /**
     * prints the final results to the standard output
     *
     * @param results results
     */
    public static void printResults(Results results) {
        printResults(results, System.out);
    }

    /**
     * Prints the final results to the provided output
     *
     * @param results results
     * @param out     output stream
     */
    public static void printResults(Results results, OutputStream out) {
        if (results != null) {
            XMLSerializer xmlser = new XMLSerializer(getXMLFormatter());
            PrintWriter writer = new PrintWriter(out);
            xmlser.setOutputCharStream(writer);
            try {
                results.marshal(xmlser.asContentHandler());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }

    }

    /**
     * Prints individual result to the provided output stream
     *
     * @param result result
     * @param out    output stream
     */
    public static void printResult(Result result, OutputStream out) {
        if (result != null) {
            XMLSerializer xmlser = new XMLSerializer(getXMLFormatter());
            PrintWriter writer = new PrintWriter(out);
            xmlser.setOutputCharStream(writer);
            try {
                result.marshal(xmlser.asContentHandler());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * Prints the price of the option
     */
    public void printPrice() {
        printPrice(this.result.getPrice());
    }

    /**
     * Prints the given price element on the standard output
     *
     * @param price price element
     */
    public static void printPrice(Price price) {
        printPrice(price, System.out);
    }

    /**
     * Prints the price on the output stream
     *
     * @param price price
     * @param out
     */
    public static void printPrice(Price price, OutputStream out) {
        if (price != null) {
            XMLSerializer xmlser = new XMLSerializer(getXMLFormatter());
            PrintWriter writer = new PrintWriter(out);
            xmlser.setOutputCharStream(writer);
            try {
                price.marshal(xmlser.asContentHandler());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * Prints the computational metrics on the standard output
     */

    public void printMetrics() {
        printMetrics(this.result.getMetrics());
    }

    /**
     * Prints the provided computational metrics on the standard output
     *
     * @param metrics
     */
    public static void printMetrics(Metrics metrics) {
        printMetrics(metrics, System.out);
    }

    /**
     * Prints the computational metrics on the provided output stream
     *
     * @param metrics computational metrics
     * @param out     output stream
     */
    public static void printMetrics(Metrics metrics, OutputStream out) {
        if (metrics != null) {
            XMLSerializer xmlser = new XMLSerializer(getXMLFormatter());
            PrintWriter writer = new PrintWriter(out);
            xmlser.setOutputCharStream(writer);
            try {
                metrics.marshal(xmlser.asContentHandler());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * Prints the greeks on the standard output
     */

    public void printGreeks() {
        printGreeks(this.result.getGreeks());
    }

    /**
     * Prints the given greeks on the standard output
     *
     * @param greeks computed greek values
     */
    public static void printGreeks(Greeks greeks) {
        printGreeks(greeks, System.out);
    }

    /**
     * Prints the greek values on the provided output stream
     *
     * @param greeks computed greek values
     * @param out    output stream
     */

    public static void printGreeks(Greeks greeks, OutputStream out) {
        if (greeks != null) {
            XMLSerializer xmlser = new XMLSerializer(getXMLFormatter());
            PrintWriter writer = new PrintWriter(out);
            xmlser.setOutputCharStream(writer);
            try {
                greeks.marshal(xmlser.asContentHandler());
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    /**
     * creates the format of the output xml descriptor file
     *
     * @return
     */
    protected static OutputFormat getXMLFormatter() {
        OutputFormat format = new OutputFormat();
        format.setLineWidth(65);
        format.setIndenting(true);
        format.setIndent(2);
        format.setOmitXMLDeclaration(true);
        return format;
    }

}
